/*
 * getresponse.c
 *
 * Contains functions to obtain and interpret responses to probe
 * packets.
 *
 * Copyright (c) 2002 Todd MacDermid <tmacd@synacklabs.net>
 *
 */

#define PACKETWAIT 300 /* In milliseconds */
#define TIMEOUT 3000 /* In milliseconds */

#include <sys/time.h>
#include "lsrscan.h"
#include "config.h"

int checkPacket(uint8_t *packetbuf, struct packetInfo *packetInfo) {
  struct ip_hdr *ipHeader;
  struct tcp_hdr *tcpHeader;
  int i;
  
  ipHeader = (struct ip_hdr *)packetbuf;
  if(ipHeader->ip_p == IPPROTO_TCP)
    tcpHeader = (struct tcp_hdr *)(packetbuf+(4*ipHeader->ip_hl));
  else 
    return(0);

  if(packetInfo->type == TYPE_THROUGH) {
    if((htonl(tcpHeader->th_seq) == htonl(packetInfo->sequenceNum)) &&
       (ipHeader->ip_src == packetInfo->ipHops[0]) &&
       (ipHeader->ip_dst == packetInfo->ipHops[packetInfo->myHop]) &&
       (tcpHeader->th_sport == htons(packetInfo->sourcePort)) &&
       (tcpHeader->th_dport == htons(packetInfo->destPort)))
      return(1);
  }
  if(packetInfo->type == TYPE_TO) {
    if((htonl(tcpHeader->th_ack) == (htonl(packetInfo->sequenceNum) + 1)) &&
       (ipHeader->ip_src == packetInfo->ipHops[packetInfo->hopCount-1]) &&
       (ipHeader->ip_dst == packetInfo->ipHops[packetInfo->myHop]) &&
       (tcpHeader->th_sport == htons(packetInfo->destPort)) &&
       (tcpHeader->th_dport == htons(packetInfo->sourcePort)))
      return(1);
  }
  return(0);
}

int getResponse(pcap_t *pcapPtr, struct packetInfo *packetInfo) {
  struct timeval start,current;
  uint8_t *packetbuf;
  int retVal;
  int linkHeadSize;
  struct pcap_pkthdr packetHead;

  if((linkHeadSize = getLinkSize(pcapPtr)) < 0) {
    fprintf(stderr, "Unsupported link type\n");
    return(-1);
  }

  gettimeofday(&start, NULL);
  gettimeofday(&current, NULL);

  while((((current.tv_sec - start.tv_sec) * 1000) + 
	 ((current.tv_usec - start.tv_usec) / 1000)) < TIMEOUT) {
    packetbuf = NULL;
    while((packetbuf = (uint8_t *)pcap_next(pcapPtr, &packetHead)) != NULL) {
      packetbuf += linkHeadSize;
      retVal = checkPacket(packetbuf, packetInfo);
      if(retVal == 1) return(1);
    }
    usleep(PACKETWAIT*1000);
    gettimeofday(&current, NULL);
  }
  return(0); 
}
