#define _BSD_SOURCE 1
#define TYPE_TO      1
#define TYPE_THROUGH 2
#define TYPE_BOTH    3

#include <dnet.h>
#include <pcap.h>

/* a settings structure contains all the information gleaned from
 * the command line options
 */

struct settings {
  uint16_t destPort;   /* Default is 80 */
  uint16_t sourcePort; /* Starts as zero. If not set in options, 
			  will be random */
  uint32_t beforeHops[8];
  uint32_t afterHops[8];
  int numBefore;
  int numAfter;
  uint32_t sourceIP; /* Starts as zero. If not set in options, 
			will be random */
  uint32_t silentIP; /* Starts as zero. If not set in options, 
			real IP will be used to sniff on. */
  uint32_t myIP;
  int scanType;
  int verbose;
  pcap_t *pcapPtr;
};

/* The packetInfo structure contains information about the probe packets.
 * It tells the sendLSR function what to send, and the getResponse
 * function what to look for as a response.
 */

struct packetInfo {
  uint16_t destPort;
  uint16_t sourcePort;
  uint32_t ipHops[11]; /*Can put 9 source route hops, source, and dest */
  int currentHop;       /*The addr we claim it just came from*/ 
  int myHop;            /*Which IP "I" am */
  int hopCount;         /*Total # of IPs in ipHops array*/
  uint32_t sequenceNum;
  struct addr myAddr;
  int type;
};

/* Function declarations */

int getLinkSize(pcap_t *);
pcap_t * initPcap(struct settings *);
int getResponse(pcap_t *, struct packetInfo *);
int sendLSR(struct packetInfo *);
