/*
 * rand.c
 *
 * Contains wrapper to the best rand easily available on the system
 *
 * Copyright (c) 2002 Todd MacDermid <tmacd@synacklabs.net>
 *
 */

#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

/* fillBufRandomly - used to fill in a buffer with as good a source of
 * entropy as is easily available. Takes in a pointer to the buffer and
 * the number of bytes to fill. Returns 0 on success, -1 on failure
 */

int fillBufRandomly(void *buf, int bytes) {
  static int seeded = 0;
  int randomFile;
  int bytesRemaining;
  struct timeval tv;
#ifdef HAVE_RAND48
  long int tempRand;
#else
  int tempRand;
#endif
  ssize_t bytesRead;

  bytesRemaining = bytes;

  if(bytes < 1) return(-1);

  if((randomFile = open("/dev/urandom", O_RDONLY)) < 0) {
    if((randomFile = open("/dev/random", O_RDONLY)) < 0) {
      if(seeded == 0) {
	gettimeofday(&tv, NULL);
#ifdef HAVE_RAND48
	srand48(tv.tv_sec ^ tv.tv_usec);
#else
	srand(tv.tv_sec ^ tv.tv_usec);
#endif
	seeded = 1;
      }
      while(bytesRemaining > 0) {
#ifdef HAVE_RAND48
	tempRand = lrand48();
#else
	tempRand = rand();
#endif
	if (bytesRemaining < sizeof(long int)) {
	  memcpy(buf, &tempRand, bytesRemaining);
	  bytesRemaining = 0;
	} else {
	  memcpy(buf, &tempRand, sizeof(long int));
	  bytesRemaining -= sizeof(long int);
	  buf += sizeof(long int);
	}
      }
      return(0);
    } 
  }
    
  while(bytesRemaining > 0) {
    if((bytesRead = read(randomFile, buf, bytesRemaining)) <= 0) {
      return(-1);
    }
    bytesRemaining -= bytesRead;
    buf+= bytesRead;
  }
  close(randomFile);
  return(0);
}

