#include <string.h>
#include <stdio.h>
#include <sys/time.h>
#include <unistd.h>
#include <dnet.h>
#include <pcap.h>

#define FQDNMAX 8192
#define TMPBUFSIZE 1024
#define FILTERSIZE 8192
#define PACKETSIZE 65535
#define PACKETWAIT 300 

int getLinkSize(pcap_t *pcapPtr) {
  int linkType;
  
  linkType = pcap_datalink(pcapPtr);
  
  if(linkType == DLT_NULL) return(4);
  else if (linkType == DLT_EN10MB) return(14);
  else if (linkType == DLT_IEEE802) return(22);
  else if (linkType == DLT_RAW) return (0);
  else if (linkType == DLT_FDDI) return (21);
  else return(-1); /* Unsupported as of yet...*/
} 

pcap_t * initPcap(struct addr *silentAddr, struct addr *remoteAddr, 
		  struct addr *spoofedAddr, struct addr *myAddr) {
  char errbuf[PCAP_ERRBUF_SIZE];
  char silentString[FQDNMAX];
  char remoteString[FQDNMAX];
  char spoofedString[FQDNMAX];
  char myString[FQDNMAX];
  char *defaultInterface;
  uint8_t buf[TMPBUFSIZE];
  pcap_t *pcapPtr;
  intf_t *localInterface;
  struct intf_entry *intfEntry;
  char filter[FILTERSIZE];
  bpf_u_int32 myNetmask, addrThing;
  int ret;
  struct bpf_program filterCode;


  if((defaultInterface = pcap_lookupdev(errbuf)) == NULL) {
    fprintf(stderr, "pcap_lookupdev failed: %s\n", errbuf);
    return(NULL);
  }

  if((localInterface=intf_open()) == NULL) {
    fprintf(stderr, "intf_open failed\n");
    return(NULL);
  }
  
  myAddr->addr_type = ADDR_TYPE_IP;
  memset(buf, 0, TMPBUFSIZE);
  intfEntry = (struct intf_entry *)buf;    
  strncpy(intfEntry->intf_name, defaultInterface, 59);
  intfEntry->intf_len = TMPBUFSIZE;
    
  if(intf_get(localInterface, intfEntry) < 0) {
    fprintf(stderr, "intf_get failed\n");
    intf_close(localInterface);
    return(NULL);
  }
  intf_close(localInterface);
  
  memcpy(&(myAddr->addr_ip), &(intfEntry->intf_addr.addr_ip), IP_ADDR_LEN);
  myAddr->addr_bits = 32;

  addr_ntop(silentAddr, silentString, FQDNMAX);

  addr_ntop(remoteAddr, remoteString, FQDNMAX);

  addr_ntop(spoofedAddr, spoofedString, FQDNMAX);

  addr_ntop(myAddr, myString, FQDNMAX);

  if((pcap_lookupnet(defaultInterface, &addrThing, &myNetmask, errbuf)) < 0) {
    fprintf(stderr, "pcap_lookupnet failed: %s\n", errbuf);
    return(NULL);
  }  

  if((pcapPtr = pcap_open_live(defaultInterface, PACKETSIZE, 1, PACKETWAIT,
			       errbuf)) == NULL) {
    fprintf(stderr, "pcap_open_live failed: %s\n",errbuf);
    return(NULL);
  }

  snprintf(filter, sizeof(filter)-1, "tcp and host %s", silentString);

  if ((pcap_compile(pcapPtr, &filterCode, filter, 1, myNetmask)) < 0) {
    fprintf(stderr, "pcap_compile failed: %s\n", pcap_geterr(pcapPtr));
    pcap_close(pcapPtr);
    return(NULL);
  }

  ret = pcap_setfilter(pcapPtr, &filterCode);
  pcap_freecode(&filterCode);
  if(ret < 0) {
    fprintf(stderr, "pcap_setfilter failed: %s\n", pcap_geterr(pcapPtr));
    pcap_close(pcapPtr);
    return(NULL);
  }
  return(pcapPtr);
}
