#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>

#include "net.h"

/* 
 * reads a line from a socket (up to max bytes).  
 * if a timeout occurs, returns -2, otherwise returns
 * the same value as read().
 */

int
sock_readline (int sock, void *buf, size_t bytes)
{
    int r, s_flags, total = 0;
    fd_set rset;
    struct timeval to;
    char *p, c;

    memset(&to, 0, sizeof(to));
    to.tv_sec = NET_TIMEOUT;

    s_flags = fcntl(sock, F_GETFL, 0);
    fcntl(sock, F_SETFL, s_flags | O_NONBLOCK); 

    p = buf;   
 
    while (total < bytes)
    {
        FD_ZERO(&rset);
        FD_SET(sock, &rset);
        select(sock + 1, &rset, NULL, NULL, &to);

        if (FD_ISSET(sock, &rset))
        {
            r = read(sock, &c, 1);

            switch(r)
            {
            case 1:
                total++;
                *p = c;
                p++;
                if (c == '\n')
                {
                    fcntl(sock, F_SETFL, s_flags);
                    return total;
                }
                break;

            default: 
                perror("sock_readline()");
                fcntl(sock, F_SETFL, s_flags);
                return r;
            }
        }
        else  /* timeout */
        {
            printf("sock_readline() timeout!\n");
            fcntl(sock, F_SETFL, s_flags);
            return -2;
        }
    }

    fcntl(sock, F_SETFL, s_flags);
    return -1;
}
        
        
void
sock_printf (int fd, char *fmt, ...)
{
    int n = dup (fd);
    FILE *stream;
    va_list ape;

    if (fd < 0 || n < 0 || fmt == NULL)
    {
        errno = EINVAL;
        return;
    }

    stream = fdopen (n, "w");
    if (stream == NULL)
        return;

    va_start (ape, fmt);
    vfprintf (stream, fmt, ape);
    va_end (ape);
    fflush (stream);
    fclose (stream);
    return;
}


