/* 
   SMB Auditing Tool
   Copyright (C) Patrik Karlsson 2001
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "config.h"
#include "smblib.h"
#include "smb.h"
#include "netbios.h"
#include "crypto.h"
#include "constants.h"
#include "util.h"
#include <stdlib.h>
#include <netinet/in.h>
#include <arpa/inet.h>

void SMBGetLoginLongError(BRUTEINFO *info, char *error) {

  UCHAR nError = info->error[0];

  if ( SMB_ERR_CHANGE_PASSWORD_NEXT_LOGIN == nError )
    strcpy(error, "User must change password at next logon");
  else if ( SMB_ERR_ACCOUNT_DISABLED == nError )
    strcpy(error, "Account is disabled");
  else if ( SMB_ERR_ACCESS_DENIED == nError )
    strcpy(error, "Acces denied");
  else if ( SMB_ERR_ACCOUNT_LOCKED_OUT == nError )
    strcpy(error, "Account has been locked out");
  else
    sprintf(error, "Error %X occured", nError);

}

void SMBInitialize(BRUTEINFO *info) {

  memset(info->server, 0, sizeof(info->server));
  memset(info->ip, 0, sizeof(info->ip));
  memset(info->workgroup, 0, sizeof(info->workgroup));
  memset(info->NativeOS, 0, sizeof(info->NativeOS));
  memset(info->domainsid, 0, sizeof(info->domainsid));
  memset(info->pipe, 0, sizeof(info->pipe));
  info->userlist = NULL;
  info->pwdlist = NULL;
  info->nMaxlogins = -1;
  info->protocol = WIN_NETBIOSMODE;
  info->timeout = 300 * 1000; /* timeout 300ms */

}

int SMBLogOff(BRUTEINFO *info) {

  UCHAR b[43] = { 0x00, 0x00, 0x00, 0x27, 0xff, 0x53, 0x4d, 0x42,
		  0x74, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7f, 0x3b,
		  0x00, 0x08, 0x01, 0x00, 0x02, 0xff, 0x00, 0x00,
		  0x00, 0x00, 0x00 };


  info->data  = (UCHAR *) malloc (1024);
  info->cmd = SMB_CMD_CLOSE;
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));
  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  /* memcpy(info->data+37, info->tid, 2); */

  info->length = sizeof(b);

  SMBSend(info);
  free(info->data);

  if ( SMBRecv(info) != TRUE )
    return FALSE;

  return TRUE;

}

int SMBClose(BRUTEINFO *info) {

  UCHAR b[45] = { 0x00, 0x00, 0x00, 0x29, 0xff, 0x53, 0x4d, 0x42,
		  0x04, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x9b, 0x37,
		  0x00, 0x08, 0x01, 0x00, 0x03, 0xFF, 0xFF, 0xff,
		  0xff, 0xff, 0xff, 0x00, 0x00 };

  /* nothing to do here we have nothing open */
  if ( info->pipe[0] == 0 && info->pipe[1] == 0 )
    return TRUE;

  info->data  = (UCHAR *) malloc (1024);
  info->cmd = SMB_CMD_CLOSE;
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));
  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+37, info->pipe, 2);

  info->length = sizeof(b);

  SMBSend(info);
  free(info->data);

  if ( SMBRecv(info) != TRUE )
    return FALSE;

  return TRUE;


}

int LSASidToUser(BRUTEINFO *info, int nSID) {


  UCHAR b[144] = { 0x00, 0x00, 0x00, 0x8c, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x07, 0x04,
		   0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x40, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x00, 0x40, 0x49, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		   0x40, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00,
		   0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x12, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x32, 0x6c, 0x81, 0x95,
		   0x6c, 0x99, 0xd5, 0x11, 0xb7, 0x72, 0x00, 0x10,
		   0x5a, 0x38, 0x17, 0x18, 0x01, 0x00, 0x00, 0x00,
		   0xe8, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x01, 0x00, 0x00, 0x00, 0xf4, 0x01, 0x00, 0x00 };

  UCHAR uSID[2];
  char* user = NULL;

  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);
  
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));
  memcpy(info->data+144, info->domainsid, sizeof(info->domainsid));

  /* uglycodeRus :) */
  uSID[1] = nSID >> 8;
  uSID[0] = nSID - ( uSID[1] * 256 );
  
  memcpy( info->data+140, uSID, sizeof(uSID));

  memcpy(info->data+108, info->lsaid, 2);

  info->length = sizeof(b);
  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE )
    return FALSE;

  /* the sid doesn't seem to exist */
  if ( info->rcvlen == 124 )
    return FALSE;

  /* stupid little test :) */
  if ( info->rbuf[112] < 255 )
    user = (char *) malloc(info->rbuf[112]*2);

  memcpy(user, info->rbuf+116, info->rbuf[112] * 2);
  user = (char *)deflate(user, info->rbuf[112] * 2);

  /* looks like it an XP server :) */
  if ( strcmp(user, "\237\b+H\002") == 0 ) {
    return FALSE;
  }

  memset(user+info->rbuf[112], 0, 1);

  info->userlist = (SMBUSERS *)ll_adduser(info->userlist, nSID, user);

  return TRUE;

}


int LSARetrieveUserInfo(BRUTEINFO *info) {

  UCHAR b[136] = { 0x00, 0x00, 0x00, 0x84, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x5b, 0x6b,
		   0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x38,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x38, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x00, 0x08, 0x41, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		   0x38, 0x00, 0x00, 0x00, 0x0a, 0x00, 0x00, 0x00,
		   0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x0d, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0xdc, 0x50, 0x64, 0xac,
		   0xd5, 0x5f, 0xd5, 0x11, 0xa3, 0xc9, 0xb3, 0x7e,
		   0x55, 0xc1, 0xc6, 0xa3, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00};

  int offset;
  int count;
  int userlen;
  char tmpuser[40];
  int i;
  SMBUSERS *smbusers;

  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);
  
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));
  memcpy(info->data+144, info->domainsid, sizeof(info->domainsid));

  /* dont ask >:) */
  info->lsaid[0]--;

  memcpy(info->data+108, info->lsaid, 2);

  info->length = sizeof(b);
  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE )
    return FALSE;

  count = info->rcvlen;
  offset = info->rbuf[51]; 
  i = 0;

  while ( offset+i<count ) {
    if ( info->rbuf[offset+i] == 0x10 && \
	 info->rbuf[offset+i+1] == 0x00 && \
	 info->rbuf[offset+i+2] == 0x00 && \
	 info->rbuf[offset+i+3] == 0x00 && \
	 info->rbuf[offset+i+4] == 0x00 && \
	 info->rbuf[offset+i+5] == 0x00 )
      break;
    else
      i++;
  }

  /* Guess restrict anonymous is set to 1 */
  if ( offset+i == count )
    return FALSE;

  offset += i + 8;

  info->userlist = NULL;

  /* loop through users */
  while ( offset < count ) {
    userlen = info->rbuf[offset];

    offset += 4;
    memset(tmpuser, 0, sizeof(tmpuser));
    memcpy(tmpuser, info->rbuf+offset, userlen*2);
    offset += userlen*2;

    memcpy(tmpuser, (char *)deflate(tmpuser,sizeof(tmpuser)),sizeof(tmpuser));

    /* linked list of users */
    smbusers = (SMBUSERS *) malloc(sizeof(SMBUSERS));
    memset(smbusers->item, 0, sizeof(smbusers->item));
    memcpy(smbusers->item, tmpuser, strlen(tmpuser));
    
    smbusers->next = info->userlist;
    info->userlist = smbusers;

    /* FIXME: fix this ugly code! */
    while ( info->rbuf[offset] != 0x10 ) {
      if ( offset < count )
	offset++;
      else 
	break;
    }
    
    offset += 8;

  }

  return TRUE;

}

int LSAOpenPolicySetupPart4(BRUTEINFO *info) {

  UCHAR b[144] = { 0x00, 0x00, 0x00, 0x8c, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x5b, 0x6b,
		   0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x40, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x00, 0x08, 0x49, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		   0x40, 0x00, 0x00, 0x00, 0x09, 0x00, 0x00, 0x00,
		   0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0xdb, 0x50, 0x64, 0xac,
		   0xd5, 0x5f, 0xd5, 0x11, 0xa3, 0xc9, 0xb3, 0x7e,
		   0x55, 0xc1, 0xc6, 0xa3, 0x04, 0x03, 0x00, 0x00,
		   0x01, 0x00, 0x00, 0x00, 0x01, 0x01, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x05, 0x20, 0x00, 0x00, 0x00};

  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);
  
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));
  memcpy(info->data+144, info->domainsid, sizeof(info->domainsid));

  /* dont ask >:) */
  info->lsaid[0]--;

  memcpy(info->data+108, info->lsaid, 2);
  info->length = sizeof(b);

  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE )
    return FALSE;

  /* store the lsa id */
  memcpy(info->lsaid, info->rbuf+88, 2);

  return TRUE;
}


int LSAOpenPolicySetupPart3(BRUTEINFO *info) {

  UCHAR b[156] = {0x00, 0x00, 0x00, 0x98, 0xff, 0x53, 0x4d, 0x42, 
		  0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x5b, 0x6b,
		  0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x4c,
		  0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x4c, 0x00, 0x4c, 0x00, 0x4c, 0x00, 0x02,
		  0x00, 0x26, 0x00, 0x00, 0x40, 0x55, 0x00, 0x5c,
		  0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		  0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		  0x4c, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		  0x34, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0xdb, 0x50, 0x64, 0xac,
		  0xd5, 0x5f, 0xd5, 0x11, 0xa3, 0xc9, 0xb3, 0x7e,
		  0x55, 0xc1, 0xc6, 0xa3, 0x04, 0x03, 0x00, 0x00,
		  0x04, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x05, 0x15, 0x00, 0x00, 0x00,
		  0x28, 0x35, 0x48, 0x58, 0x89, 0x13, 0x69, 0x02,
		  0xa3, 0x67, 0xcc, 0x74}; 
  
  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);
  
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));
  memcpy(info->data+144, info->domainsid, sizeof(info->domainsid));

  info->length = sizeof(b);

  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE ) {
    return FALSE;
  }

  /* store the lsa id */
  memcpy(info->lsaid, info->rbuf+88, 2);

  return TRUE;

}

int LSAOpenPolicySetupPart5(BRUTEINFO *info) {

  UCHAR b[156] = {0x00, 0x00, 0x00, 0x98, 0xff, 0x53, 0x4d, 0x42, 
		  0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x5b, 0x6b,
		  0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x4c,
		  0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x4c, 0x00, 0x4c, 0x00, 0x4c, 0x00, 0x02,
		  0x00, 0x26, 0x00, 0x00, 0x40, 0x55, 0x00, 0x5c,
		  0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		  0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		  0x4c, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		  0x34, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0xdb, 0x50, 0x64, 0xac,
		  0xd5, 0x5f, 0xd5, 0x11, 0xa3, 0xc9, 0xb3, 0x7e,
		  0x55, 0xc1, 0xc6, 0xa3, 0x00, 0x00, 0x00, 0x02,
		  0x04, 0x00, 0x00, 0x00, 0x01, 0x04, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x05, 0x15, 0x00, 0x00, 0x00,
		  0x28, 0x35, 0x48, 0x58, 0x89, 0x13, 0x69, 0x02,
		  0xa3, 0x67, 0xcc, 0x74}; 
  
  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);
  
  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));
  memcpy(info->data+144, info->domainsid, sizeof(info->domainsid));

  info->length = sizeof(b);

  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE ) {
    return FALSE;
  }

  /* store the lsa id */
  memcpy(info->lsaid, info->rbuf+88, 2);

  return TRUE;

}

int LSAOpenPolicySetupPart2(BRUTEINFO *info) {

  int i;
  int pad;
  char *pIP;

  /* set the SMB command to perform */
  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;

  info->data  = (UCHAR *) malloc (1024);
  pIP = ( char * ) malloc (50);

  memset(info->data, 0, 1024);
  memcpy(info->data, LSA_PIPE_2, sizeof(LSA_PIPE_2));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);

  memset(pIP, 0, 50);

  /* add zeroes between each char in ip */
  for ( i = 0; i<sizeof(info->ip); i++ ) {
    memset(pIP+(i*2), info->ip[i], 1);
  }

  memcpy(info->data+124, pIP, sizeof(info->ip) * 2);

  /* check how many bytes whe should pad */
  if ( strlen(info->ip) % 2 == 0 )
    pad = 7;
  else
    pad = 5;

  memset(info->data+124+strlen(info->ip)*2, 0x00, pad);
  memset(info->data+124+strlen(info->ip)*2+pad, 0x02, 1);

  memset(info->data+96, strlen(info->ip)+0x27, 1);
  memset(info->data+96, 0x34, 1);
  memset(info->data+39, 45+(strlen(info->ip)*2)+pad, 1);
  memset(info->data+59, 45+(strlen(info->ip)*2)+pad, 1);
  memset(info->data+69, 54+(strlen(info->ip)*2)+pad, 1);
  memset(info->data+88, 45+(strlen(info->ip)*2)+pad, 1);
  memset(info->data+108, strlen(info->ip)+3, 1);
  memset(info->data+116, strlen(info->ip)+3, 1);

  info->length = 125 + ( strlen(info->ip) * 2 ) + pad;
  memset(info->data + NB_OFF_LENGTH, 121 + (strlen(info->ip)*2) + pad, 1);
  
  SMBSend( info );
  free(info->data);
  free(pIP);

  if ( SMBRecv( info ) != TRUE ) {
    return FALSE;
  }

  /* Copy the myskoid from response block :) */
  memcpy(info->myskoid, info->rbuf+88, sizeof(info->myskoid));
  
  /* store the lsa id */
  memcpy(info->lsaid, info->rbuf+88, 2);

  return TRUE;
}

int LSAQueryDomainSID(BRUTEINFO *info) {

  UCHAR b[140] = { 0x00, 0x00, 0x00, 0xa0, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x05, 0x08, 0x1a, 0x18,
		   0x01, 0x10, 0x01, 0x00, 0x10, 0x00, 0x00, 0x54,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x54, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x05, 0x40, 0x51, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		   0x54, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00,
		   0x30, 0x00, 0x00, 0x00, 0x00, 0x00, 0x05, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x90, 0x9f, 0x77, 0xb4,
		   0xa4, 0x8c, 0xd5, 0x11, 0xa8, 0x1a, 0x00, 0x10,
		   0x4b, 0xb4, 0xe3, 0x19, 0x06, 0x00, 0x06, 0x00,
		   0x01, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00};
  
  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;
  info->data  = (UCHAR *) malloc (1024);

  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memset(info->data+NB_OFF_LENGTH, sizeof(b) + info->tmplen-1, 1);

  /* set some length */
  memset(info->data+39, 0x42+strlen(info->server)*2, 1);
  memset(info->data+59, 0x42+strlen(info->server)*2, 1);
  memset(info->data+88, 0x42+strlen(info->server)*2, 1);
  memset(info->data+124, strlen(info->server)*2, 1);
  memset(info->data+126, strlen(info->server)*2, 1);
  memset(info->data+132, strlen(info->server), 1);

  /*  memcpy(info->data+108, info->lsaid, 2); */
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));

  /* set the strange id .... */
  memcpy(info->data+140, info->tmp, info->tmplen);

  info->length = sizeof(b) + info->tmplen + 3;
  SMBSend( info );
  free(info->data);

  if (  SMBRecv( info ) != TRUE )
    return FALSE;

  memcpy(info->domainsid, info->rbuf+104, sizeof(info->domainsid) );


  return TRUE;

}

int LSAAskForComputerName(BRUTEINFO *info) {

  UCHAR b[132] = { 0x00, 0x00, 0x00, 0x80, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40, 
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x28, 0x0a,
		   0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x34,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x34, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x00, 0x40, 0x3d, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x00, 0x03, 0x10, 0x00, 0x00, 0x00,
		   0x34, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00,
		   0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x10, 0x60, 0x32, 0x5a,
		   0x1f, 0x8c, 0xd5, 0x11, 0xb7, 0x64, 0x00, 0x10,
		   0x5a, 0x38, 0x17, 0x18, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x01, 0x00};

  int i;
  int len;

  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;

  info->data  = (UCHAR *) malloc (1024);

  memset(info->data, 0, 1024);
  memcpy(info->data, b, sizeof(b));

  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);
  memcpy(info->data+108, info->myskoid, sizeof(info->myskoid));

  info->length = sizeof(b);
  SMBSend( info );
  free(info->data);
  
  if ( SMBRecv( info ) != TRUE )
    return FALSE;

  i = 0;
  len = info->rbuf[136];

  if ( ( strcmp(info->server, "*SMBSERVER") == TRUE ) || 
       ( strlen(info->server) == 0 ) ) {

    memset(info->server, 0, sizeof(info->server));
   
    for ( i=0; i<len; i++ ) {
      info->server[i] = info->rbuf[140+i*2];
    }

  }

  /* copy som strange shit..... to use in query domain sid */
  memcpy(info->tmp, info->rbuf+136, info->rbuf[136]*2+3);
  info->tmplen = info->rbuf[136]*2+3;

  return TRUE;

}

int LSAOpenPolicySetup(BRUTEINFO *info) {

  UCHAR b[152] = { 0x00, 0x00, 0x00, 0x94, 0xff, 0x53, 0x4d, 0x42,
		   0x25, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40, 
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x2b, 0x09,
		   0x00, 0x08, 0x01, 0x00, 0x10, 0x00, 0x00, 0x48,
		   0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00,
		   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		   0x00, 0x4c, 0x00, 0x48, 0x00, 0x4c, 0x00, 0x02,
		   0x00, 0x26, 0x00, 0x03, 0x40/*0x08*/, 0x51, 0x00, 0x5c,
		   0x50, 0x49, 0x50, 0x45, 0x5c, 0x00, 0x00, 0x00,
		   0x05, 0x00, 0x0b, 0x00, 0x10, 0x00, 0x00, 0x00,
		   0x48, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00,
		   0x30, 0x16, 0x30, 0x16, 0x00, 0x00, 0x00, 0x00,
		   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00,
		   0x78, 0x57, 0x34, 0x12, 0x34, 0x12, 0xcd, 0xab,
		   0xef, 0x00, 0x01, 0x23, 0x45, 0x67, 0x89, 0xac,
		   0x01, 0x00, 0x00, 0x00, 0x04, 0x5d, 0x88, 0x8a,
		   0xeb, 0x1c, 0xc9, 0x11, 0x9f, 0xe8, 0x08, 0x00,
		   0x2b, 0x10, 0x48, 0x60, 0x02, 0x00, 0x00, 0x00};

  UCHAR rbuf[1024];

  /* set the SMB command to perform */
  info->cmd = SMB_CMD_LSAOPENPOLICYSETUP;

  info->data  = (UCHAR *) malloc (1024);

  memset(info->data, 0, 1024);
  memcpy(info->data, b, 152);
  memcpy(info->data+SMB_OFF_TID, info->tid, 2);
  memcpy(info->data+SMB_OFF_PIPEID, info->pipe, 2);

  info->length = sizeof(b);
  SMBSend( info );
  free(info->data);

  SMBRecv( info );

  /* store the lsa id */
  memcpy(info->lsaid, rbuf+88, 2);

  return TRUE;

}


/* FIXME: not yet tested ! */
void SMBRequestBrowseList(BRUTEINFO *info) {

  UCHAR buf[106] = {0x00, 0x00, 0x00, 0x66, 0xFF, 0x53, 0x4D, 0x42, 
		    0x25, 0x00, 0x00, 0x00, 0x00, 0x18, 0x01, 0x20,
		    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		    0x00, 0x00, 0x00, 0x00, 0xFF, 0xFF, 0x00, 0x28, 
		    0xFF, 0xFF, 0x00, 0x00, 0x0E, 0x1A, 0x00, 0x00,
		    0x00, 0x08, 0x00, 0x10, 0x27, 0x00, 0x00, 0x00,
		    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1A,
		    0x00, 0x4C, 0x00, 0x00, 0x00, 0x66, 0x00, 0x00,
		    0x00, 0x27, 0x00, 0x5C, 0x50, 0x49, 0x50, 0x45,
		    0x5C, 0x4C, 0x41, 0x4E, 0x4D, 0x41, 0x4E, 0x00,
		    0x68, 0x00, 0x57, 0x72, 0x4C, 0x65, 0x68, 0x44,
		    0x4F, 0x00, 0x42, 0x31, 0x36, 0x42, 0x42, 0x44,
		    0x7A, 0x00, 0x01, 0x00, 0x00, 0x20, 0xFF, 0xFF,
		    0xFF, 0xFF};

  UCHAR rbuf[1024];

  memcpy(info->data, buf, 106);
  SMBSend( info );
  recv(info->fd, rbuf, 1024, 0);
		  
}

int SMBNTCreateAndX(BRUTEINFO *info) {

  UCHAR b[93] = { 0x00, 0x00, 0x00, 0x59, 0xff, 0x53, 0x4d, 0x42,
		  0xa2, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0xce, 0x15,
		  0x00, 0x08, 0x01, 0x00, 0x18, 0xff, 0x00, 0x00,
		  0x00, 0x00, 0x05, 0x00, 0x06, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x9f, 0x01, 0x02, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x03, 0x00, 0x00, 0x00,
		  0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x02, 0x00, 0x00, 0x00, 0x00, 0x06, 0x00, 0x5c,
		  0x73, 0x61, 0x6d, 0x72, 0x00};

  /* set the SMB command to perform */
  info->cmd = SMB_CMD_CREATEANDX;

  info->data  = (UCHAR *) malloc (100);

  memset(info->data, 0, 100);
  memcpy(info->data, b, 93);
  memcpy(info->data+SMB_OFF_TID, info->tid, 2);

  info->length = sizeof(b);
  SMBSend( info );
  free(info->data);

  if ( SMBRecv( info ) != TRUE )
    return FALSE;

  /* get the pipe ID */
  memcpy(info->pipe, info->rbuf+42, 2);

  return TRUE;

}

/* 
   SMBNegProt

   Function: Negotiate protocol with server ...
             Actually a pseudo negotiation since the whole 
	     program counts on NTLM support :)

	     The challenge is retrieved from the answer
	     No error checking is performed i.e cross your fingers....
*/
void SMBNegProt(BRUTEINFO *info) {

    UCHAR buf[168] = {0x00, 0x00, 0x00, 0xa4, 0xff, 0x53, 0x4d, 0x42,
		      0x72, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x7d,
		      0x00, 0x00, 0x01, 0x00, 0x00, 0x81, 0x00, 0x02,
		      0x50, 0x43, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f,
		      0x52, 0x4b, 0x20, 0x50, 0x52, 0x4f, 0x47, 0x52,
		      0x41, 0x4d, 0x20, 0x31, 0x2e, 0x30, 0x00, 0x02,
		      0x4d, 0x49, 0x43, 0x52, 0x4f, 0x53, 0x4f, 0x46,
		      0x54, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f, 0x52,
		      0x4b, 0x53, 0x20, 0x31, 0x2e, 0x30, 0x33, 0x00,
		      0x02, 0x4d, 0x49, 0x43, 0x52, 0x4f, 0x53, 0x4f,
		      0x46, 0x54, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f,
		      0x52, 0x4b, 0x53, 0x20, 0x33, 0x2e, 0x30, 0x00,
		      0x02, 0x4c, 0x41, 0x4e, 0x4d, 0x41, 0x4e, 0x31,
		      0x2e, 0x30, 0x00, 0x02, 0x4c, 0x4d, 0x31, 0x2e,
		      0x32, 0x58, 0x30, 0x30, 0x32, 0x00, 0x02, 0x53,
		      0x61, 0x6d, 0x62, 0x61, 0x00, 0x02, 0x4e, 0x54,
		      0x20, 0x4c, 0x41, 0x4e, 0x4d, 0x41, 0x4e, 0x20,
		      0x31, 0x2e, 0x30, 0x00, 0x02, 0x4e, 0x54, 0x20,
		      0x4c, 0x4d, 0x20, 0x30, 0x2e, 0x31, 0x32, 0x00};

    int i, offset;

    info->cmd = SMB_CMD_NEGPROT;
    info->data = (UCHAR *) malloc(168);

    memcpy(info->data, buf, 168);
    info->length = 168;
    SMBSend( info );
    SMBRecv( info );

    offset = 81;
    i = 0;

    if ( strcmp(info->server, "*SMBSERVER") == TRUE ) {

      memset(info->workgroup, 0, sizeof(info->workgroup));

      while ( info->rbuf[offset+i*2] != 0 ) {
	info->workgroup[i] = info->rbuf[offset+i*2];
	i++;
      }

      offset += i*2 + 2;
      i = 0;

    }

    /* retrieve the challenge */
    memcpy(info->challenge, info->rbuf+73, sizeof(info->challenge));

    /* free the data buffer */
    free(info->data);
}

int isSocketWritable(int fd, int nTimeout) {

  fd_set rfds;
  struct timeval tv;
  int retval;

  FD_ZERO(&rfds);
  FD_SET(fd, &rfds);

  tv.tv_sec = 0;
  tv.tv_usec = nTimeout;

  retval = select(fd+1, NULL, &rfds, NULL, &tv);

  if ( retval )
    if ( FD_ISSET(fd, &rfds) )
      return TRUE;

  return FALSE;

}

/*
  SMBConnect

  Function: Actually connect the Socket, no more, no less...
  
  Returns: TRUE on success else FALSE.

*/
int SMBConnect(BRUTEINFO *info) {

  int fd = -1;
  struct in_addr target;
  struct sockaddr_in sock;
  int nOldFlags = -1;

  if ( inet_aton((char *)info->ip, (struct in_addr *) &target) == 0) {
    fprintf(stderr, "couldn't convert address...\n");
    return -1;
  }

  fd = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP);

  sock.sin_addr.s_addr = target.s_addr;
  sock.sin_family = AF_INET;

  if ( info->protocol == WIN2000_NATIVEMODE )
    sock.sin_port = htons(445);
  else if ( info->protocol == WIN_NETBIOSMODE )
    sock.sin_port = htons(139);

  /* host might be firewalled, let's implement some timeout stuff */
  nOldFlags = fcntl(fd, F_GETFL);
  fcntl(fd, F_SETFL, O_NONBLOCK);

  /* connecting socket */
  connect(fd, (struct sockaddr *) &sock, sizeof(sock));

  if ( isSocketWritable(fd, info->timeout) == FALSE ) {
    printf("ERROR: Connecting to host...\n");
    return FALSE;
  }

  fcntl(fd, F_SETFL, nOldFlags);

  /* end timeout code */

  info->fd = fd;

  return TRUE;

}


/*
  General SMB sending function

*/
int SMBSend(BRUTEINFO *info) {

  fd_set rfds;
  struct timeval tv;
  int retval;

  info->sess_key[1] = getpid() / 100;
  info->sess_key[0] = getpid() - (100 * info->sess_key[1]);

  if ( info->cmd != SMB_CMD_SESSIONREQUEST ) {
    /* set session key */
    memcpy(info->data+SMB_OFF_SESSIONKEY, info->sess_key, 2);
    /* set user id */
    memcpy(info->data+SMB_OFF_USERID, info->userid, 2);
  }

  if ( info->fd != -1 ) {

    FD_ZERO(&rfds);
    FD_SET(info->fd, &rfds);
  
    tv.tv_sec = 2;
    tv.tv_usec = 0;

    retval = select(info->fd+1, NULL, &rfds, NULL, &tv);

    if ( !retval )
      return FALSE;

    /* set error code to ZERO */
    memset(info->error, 0, sizeof(info->error));

    if ( send( info->fd, info->data, info->length, 0) != -1 )
      return TRUE;
    else
      return FALSE;
  }

  return FALSE;
}

int SMBRecv(BRUTEINFO *info) {

  fd_set rfds;
  struct timeval tv;
  int retval;

  if ( info->fd != -1 ) {

    FD_ZERO(&rfds);
    FD_SET(info->fd, &rfds);
  
    tv.tv_sec = 2;
    tv.tv_usec = 0;

    retval = select(info->fd+1, &rfds, NULL, NULL, &tv);
    
    if ( (! retval) || (retval == -1) )
      return FALSE;

    info->rcvlen = recv(info->fd, info->rbuf, sizeof(info->rbuf), 0);
    
    /* do we have an error ? */
    if ( info->rbuf[9] != 0 )
      return FALSE;

    return TRUE;
  }

  return FALSE;

}

/* 
   NBS Session Request

   Function: Request a new session from the server

   Returns: TRUE on success else FALSE.

*/
int NBSSessionRequest(BRUTEINFO *info) {

  char nb_name[32]; /* netbiosname */
  char nb_local[32]; /* netbios localredirector */

  UCHAR rqbuf[12] = {0x81, 0x00, 0x00, 0x48, 0x20, 0x00, 0x20, 0x00,
			   0x00, 0x00, 0x00, 0x00};
  
  UCHAR rbuf[5];
  char *sRemote;

  /* if we are running in native mode aka port 445 dont do netbios */
  if ( info->protocol == WIN2000_NATIVEMODE )
    return TRUE;
  
  info->cmd = SMB_CMD_SESSIONREQUEST;
  
  if ( strlen(info->server) == 0 )
    memcpy(info->server, "*SMBSERVER", sizeof("*SMBSERVER"));

  info->data = (UCHAR *) malloc(100);
  sRemote  = (UCHAR *) malloc(100);

  memset(sRemote, 0, 100);

  memcpy(sRemote, "*SMBSERVER", sizeof("*SMBSERVER"));
  memset(sRemote+sizeof("*SMBSERVER"), 0, 1);

  memset(nb_name, 0, sizeof(nb_name));
  getNetbiosName((char *)&nb_name, sRemote);
  getNetbiosRedirectorName((char *)&nb_local);

  memset(info->data, 0, 100);

  memcpy(info->data, rqbuf, 5);
  memcpy(info->data+5, nb_name, sizeof(nb_name));
  memcpy(info->data+5+sizeof(nb_name), rqbuf+5, 2);
  memcpy(info->data+(7+sizeof(nb_name)), nb_local, sizeof(nb_local));

  info->length = (12+sizeof(nb_name)+sizeof(nb_local));
  SMBSend( info );

  recv(info->fd, rbuf, 5, 0);

  free(info->data);
  free(sRemote);

  if ( rbuf[0] == 130 )
    return TRUE;

  return FALSE;

}

/*
   SMB Tree Connect 
   
   Function : Connect to the IPC$ share of the remote computer.
   
   Returns : TRUE on Success else FALSE

*/

int SMBTConX(BRUTEINFO *info) {
  
  UCHAR b[60] = { 0x00, 0x00, 0x00, 0xff, 0xff, 0x53, 0x4d, 0x42, 
		  0x75,	0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40, 
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xff, 
		  0x00, 0x08, 0x01, 0x00, 0x04, 0xff, 0x00, 0x00, 
		  0x00, 0x00, 0x00, 0x01, 0x00, 0xff, 0x00, 0x00, 
		  0x5c, 0x5c, 0x5c, 0x49, 0x50, 0x43, 0x24, 0x00, 
		  0x49, 0x50, 0x43, 0x00}; 


  info->cmd = SMB_CMD_TCONX;
  info->data  = ( UCHAR * ) malloc ( 100 );

  memset(info->data, 0, 100);

  memcpy(info->data, b, 50);
  memcpy(info->data+50, "*SMBSERVER", strlen("*SMBSERVER"));
  memcpy(info->data+50+strlen("*SMBSERVER"), b+50, 10);

  info->data[3]  = 56 + strlen("*SMBSERVER");
  info->data[45] = 13 + strlen("*SMBSERVER");  

  info->length = sizeof(b) + strlen("*SMBSERVER");
  SMBSend(info);
  SMBRecv(info);

  memcpy(info->tid, info->rbuf+SMB_OFF_TID, 2);

  free(info->data);

  return TRUE;
}

int SMBSessionSetupNullSession(BRUTEINFO *info) {

  UCHAR b[85] = {0x00, 0x00, 0x00, 0x51, 0xff, 0x53, 0x4d,
		  0x42, 0x73, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x95,
		  0x23, 0x00, 0x00, 0x01, 0x00, 0x0d, 0xff, 0x00,
		  0x00, 0x00, 0xff, 0xff, 0x02, 0x00, 0x95, 0x23,
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		  0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00,
		  0x14, 0x00, 0x00, 0x4d, 0x59, 0x47, 0x52, 0x4f,
		  0x55, 0x50, 0x00, 0x55, 0x6e, 0x69, 0x78, 0x00,
		  0x53, 0x61, 0x6d, 0x62, 0x61, 0x00};

  UCHAR nError;
  int i;

  info->cmd = SMB_CMD_SESSIONSETUP;
  info->data = (UCHAR *) malloc ( 85 );

  memcpy(info->data, b, 85);

  info->length = 85;
  SMBSend( info );
  free(info->data);

  SMBRecv( info );
  memcpy(info->userid, info->rbuf+32, 2);

  i = 0;
  memset(info->NativeOS, 0, sizeof(info->NativeOS));

  while ( info->rbuf[45+i] != 0 && i<sizeof(info->NativeOS) ) {
    info->NativeOS[i] = info->rbuf[45+i];
    i++;
  }
  
  if ( info->rbuf[9] == 0x00) {
    return 0;
  }
  else {
    /* get the error message */
    nError = info->rbuf[11];
    return nError;
  }

}

/*
  SMBSessionSetup

  Function: Send username + response to the challenge from
            the server.

	    Currently we're sendin ZEROES for the LMHASH since
	    NT4/2000 doesn't seem to look at this if we got a
	    valid NTLM hash.

  Returns: TRUE on success else FALSE.
*/
int SMBSessionSetup(BRUTEINFO *info) {
	       

  UCHAR b[137] = {0x00, 0x00, 0x00, 0x85, 0xff, 0x53, 0x4d, 
		  0x42, 0x73, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 
		  0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3c, 
		  0x7d, 0x00, 0x00, 0x01, 0x00, 0x0d, 0xff, 0x00, 
		  0x00, 0x00, 0xff, 0xff, 0x02, 0x00, 0x3c, 0x7d, 
		  0x00, 0x00, 0x00, 0x00, 0x18, 0x00, 0x18, 0x00, 
		  0x00, 0x00, 0x00, 0x00, 0x50, 0x00, 0x00, 0x00, 
		  0x48, 0x00, 0xdf, 0x82, 0xb9, 0x2f, 0xda, 0xdb, 
		  0x43, 0x47, 0x09, 0xdb, 0x7f, 0xfc, 0xb0, 0xa8, 
		  0xf0, 0x46, 0xe2, 0xfe, 0x64, 0x6d, 0x67, 0x58, 
		  0xe0, 0xf9, 0xca, 0x9f, 0x5e, 0xdb, 0xe0, 0x15, 
		  0x80, 0xf8, 0x54, 0xe3, 0x6e, 0xe9, 0xf8, 0x88, 
		  0x80, 0x19, 0xb6, 0xf9, 0xae, 0xb7, 0xa6, 0x62, 
		  0x14, 0xfc, 0x54, 0x45, 0x53, 0x54, 0x00, 0x4d, 
		  0x59, 0x47, 0x52, 0x4f, 0x55, 0x50, 0x00, 0x55, 
		  0x6e, 0x69, 0x78, 0x00, 0x53, 0x61, 0x6d, 0x62, 
		  0x61, 0x00};

  UCHAR *user;

  UCHAR *NTLMhash;
  UCHAR *LMhash;
  int userlen;

  info->cmd = SMB_CMD_SESSIONSETUP;

  /* do we have a null session */
  if ( ( strlen(info->user) == 0 ) && ( strlen(info->pass) == 0 ) )
    return SMBSessionSetupNullSession(info);

  info->data = (UCHAR *) malloc(512);
  NTLMhash   = (UCHAR *) malloc(24);
  user       = (UCHAR *) malloc(100);

  LMhash   = (UCHAR *) malloc(24);

  HashNTLM(&NTLMhash, info->pass, info->challenge);
  /* HashLM(&LMhash, pass, challenge); */

  memset(info->data, 0, 512);
  memcpy(info->data, b, 89);
  memcpy(info->data+65, LMhash, 24);
  memcpy(info->data+89, NTLMhash, 24);

  memset(user, 0, 100);
  strncpy(user, info->user, strlen(info->user));

  userlen = strlen(info->user);

  memcpy(info->data+113, user, userlen);
  memset(info->data+(113+userlen), 0, 1);
  memcpy(info->data+(114+userlen), info->workgroup, strlen(info->workgroup));
  memcpy(info->data+(114+userlen+strlen(info->workgroup)), b+125, 12);

  /* set the header length !? */
  info->data[3] = ( userlen + strlen(info->workgroup) + 0x7A ) % 256;
  info->data[2] = ( userlen + strlen(info->workgroup) + 0x7A ) / 256;

  /* set data length !? */
  info->data[63] = 0x1F + strlen(info->workgroup) + userlen;

  info->length = (126+userlen+strlen(info->workgroup));
  SMBSend( info );

  free(info->data);
  free(NTLMhash);
  free(user);
  free(LMhash);

  if ( SMBRecv( info ) != TRUE ) {
    /* get the error message */
    memcpy(info->error, info->rbuf+9, sizeof(info->error));
    return FALSE;
  }
  else {
    /* looks like a succesfull log in, grab the userid */
    memcpy(info->userid, info->rbuf+32, 2);
  }

  return TRUE;
  
}

/*
  SMBShutdown

  Function: Close the socket!
*/
void SMBShutdown(const BRUTEINFO *info) {
  SMBClose((BRUTEINFO *)info);
  SMBLogOff((BRUTEINFO *)info);
  close(info->fd);
}


/* Only send data ... do not recieve it */
int NBSSendSessionRequest(BRUTEINFO *info) {

  char nb_name[32]; /* netbiosname */
  char nb_local[32]; /* netbios localredirector */

  UCHAR rqbuf[12] = {0x81, 0x00, 0x00, 0x48, 0x20, 0x00, 0x20, 0x00,
			   0x00, 0x00, 0x00, 0x00};
  
  char *sRemote;

  /* if we are running in native mode aka port 445 dont do netbios */
  if ( info->protocol == WIN2000_NATIVEMODE )
    return TRUE;
  
  info->cmd = SMB_CMD_SESSIONREQUEST;
  
  if ( strlen(info->server) == 0 )
    memcpy(info->server, "*SMBSERVER", sizeof("*SMBSERVER"));

  info->data = (UCHAR *) malloc(100);
  sRemote  = (char *) malloc(100);

  memset(sRemote, 0, 100);

  memcpy(sRemote, "*SMBSERVER", sizeof("*SMBSERVER"));
  memset(sRemote+sizeof("*SMBSERVER"), 0, 1);

  memset(nb_name, 0, sizeof(nb_name));
  getNetbiosName((char *)&nb_name, sRemote);
  getNetbiosRedirectorName((char *)&nb_local);

  memset(info->data, 0, 100);

  memcpy(info->data, rqbuf, 5);
  memcpy(info->data+5, nb_name, sizeof(nb_name));
  memcpy(info->data+5+sizeof(nb_name), rqbuf+5, 2);
  memcpy(info->data+(7+sizeof(nb_name)), nb_local, sizeof(nb_local));

  info->length = (12+sizeof(nb_name)+sizeof(nb_local));
  if ( send(info->fd, info->data, info->length, 0) == -1 ) {
    free( info->data );
    free( sRemote );
    return FALSE;
  }

  free( info->data );
  free( sRemote );
  return TRUE;

}

/* Only send data ... do not recieve it */
int SMBSendNegProt(BRUTEINFO *info) {

    UCHAR buf[168] = {0x00, 0x00, 0x00, 0xa4, 0xff, 0x53, 0x4d, 0x42,
		      0x72, 0x00, 0x00, 0x00, 0x00, 0x08, 0x01, 0x40,
		      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		      0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3c, 0x7d,
		      0x00, 0x00, 0x01, 0x00, 0x00, 0x81, 0x00, 0x02,
		      0x50, 0x43, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f,
		      0x52, 0x4b, 0x20, 0x50, 0x52, 0x4f, 0x47, 0x52,
		      0x41, 0x4d, 0x20, 0x31, 0x2e, 0x30, 0x00, 0x02,
		      0x4d, 0x49, 0x43, 0x52, 0x4f, 0x53, 0x4f, 0x46,
		      0x54, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f, 0x52,
		      0x4b, 0x53, 0x20, 0x31, 0x2e, 0x30, 0x33, 0x00,
		      0x02, 0x4d, 0x49, 0x43, 0x52, 0x4f, 0x53, 0x4f,
		      0x46, 0x54, 0x20, 0x4e, 0x45, 0x54, 0x57, 0x4f,
		      0x52, 0x4b, 0x53, 0x20, 0x33, 0x2e, 0x30, 0x00,
		      0x02, 0x4c, 0x41, 0x4e, 0x4d, 0x41, 0x4e, 0x31,
		      0x2e, 0x30, 0x00, 0x02, 0x4c, 0x4d, 0x31, 0x2e,
		      0x32, 0x58, 0x30, 0x30, 0x32, 0x00, 0x02, 0x53,
		      0x61, 0x6d, 0x62, 0x61, 0x00, 0x02, 0x4e, 0x54,
		      0x20, 0x4c, 0x41, 0x4e, 0x4d, 0x41, 0x4e, 0x20,
		      0x31, 0x2e, 0x30, 0x00, 0x02, 0x4e, 0x54, 0x20,
		      0x4c, 0x4d, 0x20, 0x30, 0x2e, 0x31, 0x32, 0x00};

    info->cmd = SMB_CMD_NEGPROT;
    info->data = (UCHAR *) malloc(168);

    memcpy(info->data, buf, 168);
    info->length = 168;

    if ( send(info->fd, info->data, info->length, 0) == -1 ) {
      free(info->data);
      return FALSE;
    }

    free(info->data);
    return TRUE;
}


