#ifndef __SCANNER_POP3_H
#define __SCANNER_POP3_H

#ifndef __SCANNER_SERVER_H
#include "scanner_server.h"
#endif

#include "pop3_commands.h"

class scanner_pop3 : public scanner_server
{
private:

    /// This holds the list of return fingerprints from file
    dllist<fp_return> returnfp;
    /// This is the pointer to the return fingerprints from the last scan
    fp_return * scanned_return;
    /// Loads all the fingerprints we need
    bool load_return_fps (void);
    /// Does a login to the server
    bool do_login( );
    /// Adds a read fp to our list (called virtually from base class)
    virtual bool add_fp( fp& n_fp, fp_type f_type);
    /// Check if the scan is finished
    virtual bool scan_finished( void );
    /// Initializes the scan (connect, fp prepare, banner read)
    virtual bool scan_init( void );
    /// Is beeing called every step, with step number. If a scan wants a
    /// special behaviour at a certain point
    virtual bool scan_special( long number );
    virtual bool scan_reconnected( long number );
    /// Adds a read answer to the fp(s)
    virtual bool scan_add_to_fp( const mstring&  answer );
protected:

public:
    virtual ~scanner_pop3();
    /// We need a socket, the debuglevel, if beeing polite, connect & receive timeout
    scanner_pop3(small_socket& sk, debug_level debug, bool pol, long ct, long rt) :
	    scanner_server(sk,debug, pol,ct, rt), scanned_return(NULL)
	    {
                cmd_list = pop3_cmds;
		cmd_number = sizeof (pop3_cmds) / sizeof(*pop3_cmds);
	    };
    /// returns the protocol we are scanning
    inline virtual mstring protocol( void ) { return "POP3"; }
    /// does everything for initialization, returns false in case of error
    virtual bool initialize( void );
    /// finalize a scan (generate error values)
    virtual void finalize( void );
    /// returns all the fingerprints that will then be displayed
    virtual dllist < dllist < fp > > get_fps( void );
    /// returns the descriptions belonging to the fingerprints
    virtual dllist< mstring > get_descs( void );
    /// Does a fast probe, if the server is of this protocol
    mstring is_server( void );

};

#endif

