/**
 * $Id: mstring.cpp,v 1.35 2003/04/16 21:07:23 plasmahh Exp $
 * $Revision: 1.35 $
 * $Author: plasmahh $
 * $Date: 2003/04/16 21:07:23 $
 */

#include "mstring.h"
#include "exception.h"

uint32 mstring::checksum(checksum_type type) const/*{{{*/
{

    switch (type)
    {
	case SUM :
	    return checksum_sum();
	case CRC8 :
	case CRC16 :
	case CRC32 :
	case CHECK8 :
	case CHECK16 :
	case CHECK32 :
	case MD5 :
	case SHA1 :
	    throw x_implemented ( "algo SHA1",__FLF__);
	break;	
	default :
	    throw x_parameter ( "??? ", __FLF__);
    } // of switch
}/*}}}*/

uint32 mstring::checksum_sum( void ) const/*{{{*/
{
    uint32 sum = 0;
    for(register uint32 i=0;i < leng;i++)
    {
	sum+=(uint32) sp[i];
    }
    return sum;
}/*}}}*/
 
mstring operator + ( const mstring& a, const mstring& b)/*{{{*/
{
    char8 * ptr;
    ptr = new char8[a.leng+b.leng+1];
#ifdef _SAFE_
    if (!ptr) 
    {
	throw x_mem("operator+(char8,char8)",__FLF__);
    }
#endif
    ptr[a.leng+b.leng]='\0';
    strncpy(ptr, a.sp, a.leng);
    strncpy(ptr+a.leng,b.sp,b.leng);
    mstring g(ptr);
    delete [] ptr;
    return g;
}/*}}}*/

mstring::mstring( )/*{{{*/
{
    sp = new char8[1];
    sp[0] = '\0';
    leng=0;
}/*}}}*/

mstring::mstring( char8 c)/*{{{*/
{
    sp = new char8[2];
    sp[0] = c;
    sp[1] = 0;
    leng=1;
}/*}}}*/

mstring::mstring(const char8 * cstr)  /*{{{*/
{
    sp = NULL;
    leng = strlen(cstr);
    sp = new char8[leng+1];
#ifdef _SAFE_
    // Does this work here? if we can't get memory, what will really hapen to us ?
    if (!sp) throw x_mem("mstring::mstring(char8)",__FLF__);
#endif
    strncpy(sp, cstr, leng);
    sp[leng] = '\0';
}/*}}}*/

mstring::mstring( const mstring & v)/*{{{*/
{
    sp = NULL;
    leng = v.leng;
    sp = new char8[leng+1];
#ifdef _SAFE_
    if (!sp) throw x_mem("mstring:mstring(mstring)",__FLF__);
#endif
    strncpy(sp, v.sp, leng);
    sp[leng] = '\0';
}/*}}}*/

mstring& mstring::operator += ( const mstring& v)/*{{{*/
{
    char8 * ptr;
    ptr = new char8[leng + v.leng + 1];
#ifdef _SAFE_
    if (!ptr) throw x_mem("mstring::operator+=(mstring)",__FLF__);
#endif
    if (sp)
    {
	strncpy ( ptr, sp, leng);
    }
    delete [] sp;
    strncpy ( ptr + leng, v.sp, v.leng);
    leng += v.leng;
    sp = ptr;
    sp[leng] = '\0';
    return *this;
}/*}}}*/

mstring& mstring::operator += ( const char8 * v)/*{{{*/
{
    char8 * ptr;
    uint32 i = strlen(v);
    ptr = new char8[leng + i + 1];
#ifdef _SAFE_
    if (!ptr) throw x_mem("mstring::operator+=(char8)",__FLF__);
#endif
    if (sp)
    {
	strncpy( ptr, sp, leng);
    }
    delete [] sp;
    strncpy( ptr + leng, v, i);
    leng += i;
    sp = ptr;
    sp[leng]='\0';
    return *this;
}/*}}}*/

mstring& mstring::operator =  ( const char8 *  v)/*{{{*/
{
    if (sp != v)
    {
	delete[] sp; 
        leng  = strlen(v);
	//cout << "operator= : lenge : " << leng << " : " << v << endl;
        sp = new char8[leng + 1];
#ifdef _SAFE_
	if (!sp) throw x_mem("mstring::operator=(char8)",__FLF__);
#endif
        strncpy(sp, v, leng);
    }
    sp[leng]='\0';
    //cout << "AFTER : " << sp << endl;
    return *this;
}/*}}}*/

mstring& mstring::operator = ( const mstring & v)/*{{{*/
{
    if ( sp != v.sp )
    {
	delete[] sp; 
	sp = new char8[v.leng+1];
#ifdef _SAFE_
if (!sp) throw x_mem("mstring::operator=(mstring)",__FLF__);
#endif
	strncpy(sp, v.sp, v.leng);
	leng = v.leng;
    }
    sp[leng]='\0';
    return *this;
}/*}}}*/

char8 * mstring::operator[] (uint32 pos)/*{{{*/
{
#ifndef _FAST_
    if ( sp == NULL) return NULL;
#endif

#ifdef _SAFE_
    if ( pos >= leng ) return NULL; 
#endif

    return  & sp[pos];
}/*}}}*/

char8 mstring::operator() (uint32 pos) const/*{{{*/
{
#ifndef _FAST_
    if ( sp == NULL) return '\0';
#endif

#ifdef _SAFE_
    if (pos >= leng) return '\0';
#endif

    return sp[pos];
} /*}}}*/

bool mstring::matches( const char8 * v , uint32 pos) const/*{{{*/
{
    if ( pos > 0 )
    {
	pos--;
    }
    uint32 len;
    len = strlen(v);
#ifdef _SAFE_
    if ( pos > leng ) return false;
#endif
    return ( strncmp (sp + pos, v,len ) == 0);
}/*}}}*/

bool mstring::matches( const mstring& v , uint32 pos) const/*{{{*/
{
    if ( pos > 0 )
    {
	pos--;
    }
#ifdef _SAFE_
    if ( pos > leng ) return false;
#endif
    return ( strncmp (sp + pos, v.sp, v.leng ) == 0);
}/*}}}*/

uint32 mstring::lsearch( const char8* v, uint32 spos) const/*{{{*/
{
#ifdef _SAFE_
    if ( spos >= leng ) return 0;
#endif
    uint32 t = (uint32) strstr(sp + spos, v);
    if ( t == 0) 
    { 
	return 0; 
    }

    return t - (uint32) sp + 1;
}/*}}}*/

uint32 mstring::lsearch( const mstring& v, uint32 spos) const/*{{{*/
{
#ifdef _SAFE_
    if ( spos > leng ) return false;
#endif
    uint32 t = (uint32) strstr(sp + spos, v.sp);
    if ( t == 0 ) 
    { 
	return 0; 
    }
    return t - (uint32) sp + 1;
}/*}}}*/

mstring mstring::strip_all_digits( void )/*{{{*/
{
    if ( leng == 0) return mstring("");
    mstring stripped;
    char8 * strip;
    strip = new char8[leng+1];
    uint32 ap = 0;
    for (uint32 p=0;p < leng; p++)
    {
	switch ( sp[p] )
	{
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		continue;
	    default:
		strip[ap++] = sp[p];
	}
    }
    strip[ap] = '\0';
    stripped = strip;
    delete[] strip;
    return stripped;
 }/*}}}*/

mstring mstring::strip_all_whitespaces( void )/*{{{*/
{
    if ( leng == 0 ) return mstring("") ;
    mstring stripped;
    char8 * strip;
    strip = new char8[leng+1];
    uint32 ap = 0;
    for (uint32 p=0;p < leng; p++)
    {
	if ( sp[p] == ' ' || sp[p] == '\t' || sp[p] == '\r' || sp[p] == '\n' )
	{
	    continue;
	}
	else
	{
	    strip[ap++] = sp[p];
	}
    }
    strip[ap] = '\0';
    stripped = strip;
    delete[] strip;
    return stripped;
}/*}}}*/

mstring mstring::strip_whitespaces( void )/*{{{*/
{
    if ( leng == 0) return mstring("");
    char8 * strip;
    bool begin = false;
    strip = new char8[leng+1];
    uint32 ap = 0;
// first strip all the leading whitespaces...
    for (register uint32 p=0;p <= leng; p++)
    {
	if (!begin && ( sp[p] == ' ' || sp[p] == '\t' || sp[p] == '\r' || sp[p] == '\n' ))
	{
	    continue;
	}
	else
	{
	    begin = true;
	    strip[ap++] = sp[p];
	}
    }
    begin = false;
    ap--;
    ap--;
    for (;ap >= 0;ap--)
    {
	if( (strip[ap] == ' ' || strip[ap] ==  '\t' || strip[ap] == '\r' || strip[ap] == '\n' ))
	{
	    strip[ap] = '\0';
	    continue;
	}
	else
	{
	    break;
	}
    }
    mstring stripped(strip);
    delete[] strip;
    return stripped;
}/*}}}*/

mstring mstring::substring( uint32 anf, uint32 end) const/*{{{*/
{
    mstring sub;
    if ( anf == 0) 
    {
	anf = 1;
    }
    if ( end == 0 || end > leng)  
    { 
	end = leng; 
    }
    if ( anf > leng || anf > end ) 
    { 
	return sub; 
    } 
    char8 * nw;
    nw = new char8[end - anf + 2];
    nw[ end - anf + 1] = '\0';
    strncpy ( nw, sp + (anf - 1) , end - anf + 1 );
    sub = nw;
    delete[] nw;
    return sub;
}/*}}}*/

bool mstring::operator != ( const mstring& v)/*{{{*/
{
    if ( leng != v.leng ) 
    {
	return true;
    }
    return (strcmp(sp,v.sp) != 0);
}/*}}}*/

bool mstring::operator != ( const char8 *  v)/*{{{*/
{
    return (strcmp(sp,v) != 0);
}/*}}}*/

bool mstring::operator == ( const mstring& v)/*{{{*/
{
    if (leng != v.leng) 
    {
	return false;
    }
    return (strcmp(sp, v.sp) == 0);
}/*}}}*/

bool mstring::operator == ( const char8 *  v)/*{{{*/
{
    return (strcmp(sp,v) == 0);
}/*}}}*/

ostream& operator << ( ostream & o, const mstring& v)/*{{{*/
{
    o << v.sp;
    return o;
}/*}}}*/

mstring & mstring::format( const char8* fmtstr, ...)/*{{{*/
{
// Our problem here is, that we do not know how big the resulting string will be. We could try
// and "guess" how big it will be, but for now we will stop at ~20480 bytes, which will be enough 
// for normal use.
// The trailing \0 should be set automatgically by vsprintf, so we don't have to care bout it
 
   char8 tmp[20480 + 10]; // + 10 extra bytes, in case I  made something wrong ;)
   
   va_list   parms;  
   va_start( parms, fmtstr );
   
   vsnprintf( tmp,20480, fmtstr, parms );
   
   va_end(parms);
   
   *this = tmp;
   
   return *this;
}/*}}}*/

mstring operator * ( uint32 v, const mstring& g)/*{{{*/
{
    mstring ret;
    for (register uint32 i=0; i < v; i++)
    {
	ret = ret + g;
    }
    return ret;
}/*}}}*/

bool mstring::operator <  ( const mstring& v) const/*{{{*/
{
    return ( strcmp (sp, v.sp) < 0 );
}/*}}}*/

bool mstring::operator <  ( const char8 *  v) const/*{{{*/
{
    return ( strcmp (sp, v) < 0 );
}/*}}}*/

bool mstring::operator >  ( const mstring& v) const/*{{{*/
{
    return ( strcmp (sp, v.sp) > 0 );
}/*}}}*/

bool mstring::operator >  ( const char8 *  v) const/*{{{*/
{
    return ( strcmp (sp, v) > 0 );
}/*}}}*/

mstring mstring::replace( const mstring& search, const mstring& subst )/*{{{*/
{
    if ( leng < search.leng || leng == 0 || search.leng == 0) return *this;

    mstring replaced;

    int spos = 0;
    int bpos = 0;

    spos = lsearch(search,spos);
    while ( spos > 0 )
    {
	if (spos > 0)
	{
	    replaced += substring(bpos,spos-1);
	    replaced += subst;
	    bpos = spos + search.leng;
	}
	spos = lsearch(search,bpos-1);
    }
    replaced += substring(bpos,0);
    return replaced;
}/*}}}*/

mstring mstring::replace_numbers( const mstring& subst )/*{{{*/
{
    if ( leng == 0 ) return *this;
    if ( subst.leng == 0) return strip_all_digits();

    mstring replaced;

    uint32 spos = 0;
    bool in_number = false;

    while ( spos < leng )
    {
	switch ( (*this)(spos) )
	{
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
		if ( !in_number )
		{
		    replaced += subst;
		}
		in_number = true;
		break;
	    default:
		replaced += (*this)(spos);
		in_number = false;
		break;
	}
	spos++;
    }
    return replaced;
}/*}}}*/

mstring mstring::strip_all_from ( const mstring & to_st )/*{{{*/
{
    if (leng == 0 || leng < to_st.leng || to_st.leng == 0) return *this;
    char8 * stripped = new char [ leng + 1 ];

    memcpy ( stripped, sp, leng + 1);

    
    char8 * begin ;
    
    begin = strstr ( stripped, to_st.sp );
   /* 
    cout << "BEGIN STRIPPING : " << endl;
    cout << "STRING : \"" << stripped << "\""<< endl;
    cout << "STRIFOR: \"" << to_st <<"\"" << endl; 
    cout << "LENGTH : \"" << leng << "\""<< endl;
*/	
    while ( begin != NULL )
    {
/*
	cout << "----------------------------------------------------" << endl;
	cout << "stripped : " << stripped << endl;
	cout << "begin : " << (void*) begin << endl;
	cout << "begin : " << begin << endl;
	cout << "begin + to_st.leng : " << (void*) (begin + to_st.leng) << endl;
	cout << "begin + to_st.leng : " << (begin + to_st.leng) << endl;
*/	
	int mv = leng - (int)begin + (int)stripped - to_st.leng + 1;
//	cout << "mv : " << mv << endl;
	memmove ( begin, begin + to_st.leng , mv );
	begin = strstr ( stripped, to_st.sp );
    }
    mstring ret(stripped);
    delete[] stripped;
    return ret;

}/*}}}*/

mstring mstring::get_random_string( uint32 length )
{
    char8 * str = new char8[ length + 1];
    for ( register uint32 i = 0; i < length; i++ )
    {
	
	str[i] = random() % 26 + 97;
    }
    str[length] = '\0';
    mstring mstr(str);
    delete[] str;
    return mstr;
}
uint32 mstring::to_uint32( )/*{{{*/
{
    uint32 num;
    num = strtoul(sp,NULL,10);
    return num;
}/*}}}*/

/*
uint32 mstring::strip_leading_number( )
{
    switch ( this[0] )
}
*/
/**
 *$Log: mstring.cpp,v $
 *Revision 1.35  2003/04/16 21:07:23  plasmahh
 *binary string additions (base64)
 *
 *Revision 1.34  2003/03/29 16:59:39  plasmahh
 *new features
 *
 *Revision 1.33  2003/03/13 17:31:34  plasmahh
 *new stripping helper functions
 *
 *Revision 1.32  2003/02/25 19:50:07  plasmahh
 *lets release
 *
 *Revision 1.31  2003/02/25 16:37:28  plasmahh
 *mstring bugfix
 *mstring enhancement
 *
 *Revision 1.30  2003/01/07 13:35:41  plasmahh
 *new tree stuff
 *fixed somethings in mstring
 *begin making better socket classes
 *
 *Revision 1.29  2002/12/05 19:12:05  plasmahh
 *some fixes
 *
 *Revision 1.28  2002/11/25 23:38:23  plasmahh
 *new function for dllist
 *new exception stuff
 *
 *Revision 1.27  2002/10/31 22:34:46  plasmahh
 *mstring fix
 *
 *Revision 1.26  2002/10/22 14:01:07  plasmahh
 *implementing binary tree
 *
 *Revision 1.25  2002/10/16 23:07:21  plasmahh
 *fixed memory bugs in mstring
 *TODO : refine of exception structure
 *
 *Revision 1.24  2002/10/01 21:06:24  plasmahh
 *new math things
 *
 *Revision 1.23  2002/08/15 20:40:31  plasmahh
 *new string manipulation functions
 *
 *Revision 1.22  2002/08/12 13:15:25  plasmahh
 *removed some memory leaks
 *
 *Revision 1.21  2002/08/08 20:45:53  plasmahh
 *some slight changes
 *
 *Revision 1.20  2002/06/17 14:05:07  plasmahh
 *resolv for IPs added
 *
 *Revision 1.19  2002/06/11 14:17:05  plasmahh
 *Added operators.
 *Fixed sorting bug which destroyed origin list
 *
 *Revision 1.18  2002/06/10 22:10:54  plasmahh
 * added operators and fixed some sorting things
 *
 *Revision 1.17  2002/06/09 21:23:21  plasmahh
 *changed a few things that might be bugs
 *
 *Revision 1.16  2002/05/28 20:36:33  plasmahh
 *Added include for NetBSD compatibility
 *
 *Revision 1.15  2002/05/28 18:47:40  plasmahh
 *added more or less safe format function to the mstring class
 *uint32roduced a parser class
 *some new cool exceptions
 *
 *Revision 1.14  2002/05/23 19:48:08  plasmahh
 *removed some compiler warnings.
 *fixed one or two bugs.
 *refined include structure, for faster compiling
 *hm, what else ? don't know, I hope this works all ;)
 *
 *Revision 1.13  2002/05/16 16:50:42  plasmahh
 *fixed bug in mstrig::match
 *tested some code for fingerpruint32ing
 *
 *Revision 1.12  2002/05/15 19:41:16  plasmahh
 *fixed segfault bug in substring function
 *
 *Revision 1.11  2002/05/15 07:07:10  plasmahh
 *Added search functions to string class
 *played a bit ;)
 *
 *Revision 1.10  2002/05/14 20:52:22  plasmahh
 *Added fast lsearch function for string class. Additional functions will be implemented by request
 *
 *Revision 1.9  2002/05/14 16:31:13  plasmahh
 *Removed some compiler warnings
 *Added more code,  hopefully fast ;)
 *
 *Revision 1.8  2002/05/13 21:32:57  plasmahh
 *implemented file class & readline
 *implemented additional in mstring
 *
 *Revision 1.7  2002/05/05 20:24:36  plasmahh
 *added doygen configuration file
 *made dllist work
 *added dlliterator to dllist.h
 *made some test progams cooler ;)
 *added some exceptions...
 *anything else to do ?? *G*
 *
 *Revision 1.6  2002/04/30 21:35:17  plasmahh
 *first working exceptions
 *first working strings, not everything implemented, should also be tested !
 *
 *Revision 1.5  2002/04/30 19:34:06  plasmahh
 *fixed segfault with new gcc/g++ compilers
 *
 *Revision 1.4  2002/04/30 14:13:12  plasmahh
 *Added some #ifdefs for _SAFE_ ty checks in string memory allocations
 *
 *Revision 1.3  2002/04/29 22:59:08  plasmahh
 *implemented some parts of string class
 *
 *Revision 1.2  2002/04/29 18:57:09  plasmahh
 *humm...
 *
 *Revision 1.1  2002/04/29 18:51:29  plasmahh
 *Renamed string to mstring in order to avoid conflicts with standard libraries
 *
 *Revision 1.2  2002/04/18 23:01:15  plasmahh
 *- added [] operator and first sample use of _FAST_ and _SAFE_ defines
 *
 *Revision 1.1  2002/04/16 20:46:02  plasmahh
 *Added mstring class and test program
 *
 */
