/**
 * $Id: scanner_pop3.cpp,v 1.8 2003/03/27 10:58:17 plasmahh Exp $
 * $Revision: 1.8 $
 * $Author: plasmahh $
 * $Date: 2003/03/27 10:58:17 $
 */

#include "scanner_pop3.h"
#define LOGIN_TIME 13
scanner_pop3::~scanner_pop3()/*{{{*/
{
    returnfp.clear();
    delete scanned_return;
}/*}}}*/

bool scanner_pop3::initialize (void)/*{{{*/
{

    if ( debuglevel > 1 )
    {
        cout << "POP3 : Reading fingerprints from file..." << endl;
    }

    bool ret = true;
    ret &= load_return_fps ();

    if ( debuglevel > 1 )
    {
        cout << "POP3 : Done reading fingerprints, error : " << (ret ? "no" : "yes") << endl;
    }

    return ret;
}/*}}}*/

bool scanner_pop3::add_fp( fp& n_fp, fp_type f_type)/*{{{*/
{
    switch (f_type)
    {
	case FP_Return :
	    returnfp.add( (fp_return&)n_fp);
	    break;
	case FP_Conf :
	case FP_Rfc :
	case FP_Pop3 :
	case FP_Ftp  :
	case FP_Imap :
	case FP_Base :
	    throw x_implemented("POP3 : Invalid Fingerprint type (implementation failure ?)",__FLF__);
	    return false;
	    break;
    }
    return true;
}/*}}}*/

bool scanner_pop3::load_return_fps (void)/*{{{*/
{
        return load_fp("pop3-fingerprints-return","FP_RETURN",FP_Return,true);
}/*}}}*/

void scanner_pop3::finalize (void)/*{{{*/
{
    try 
    {
	if ( debuglevel > 1 )
	{
	    cout << "POP3 : Finalizing fingerprints ..." ;
	}

	if ( debuglevel > 0 )
	{
	    cout << endl << *scanned_return  << endl;
	}
	// iterate through the fingerprints and set error values
	dlliterator < fp_return > itr (returnfp);

	for (itr.first (); itr.valid (); itr.next ())
	{
	    itr.item ().gen_error (*scanned_return);
	}
	
	if ( debuglevel > 1 )
	{
	    cout << "POP3 : done." << endl;
	}
    }
    catch ( Exception & e )
    {
	if ( debuglevel > 0 )
	{
	    cout << "POP3 : done (with error)." << endl;
	}
    }
}/*}}}*/

dllist < dllist < fp > >scanner_pop3::get_fps (void)/*{{{*/
{
    /*
     * We return all the fingerprints here in a dllist.
     * 1. Item : dllist of all the config fingerprints we have
     * 2. Item : dllist of all the rfc fingerprints we have.
     * 3. Item : dllist of all the generic return fingerprints.
     *
     * Since we return all of them as FP, not as FP_CONF or whatever, we will only
     * get description and error values !!! Not the real fingerprint, which means
     * we can output it only in this class !!!  this needs us to get the debuglevel
     * in the future
     */
    dlliterator < fp_return > itr (returnfp);

    dllist < fp > fpr;

    for (itr.first (); itr.valid (); itr.next ())
    {
	fpr.add (itr.item ());
    }

    // all have been added, generate the "big" dllist
    dllist < dllist < fp > >allfps;
    allfps.add (fpr);
    return allfps;
    // return a list of lists of fingerprints
}/*}}}*/

dllist < mstring > scanner_pop3::get_descs (void)/*{{{*/
{
    // return a list of descriptions of the fingerprint types...
    // hardcode this here a bit...
    dllist < mstring > desc;
    desc.add ("POP3 Overall Fingerprinting");
    return desc;
}/*}}}*/

bool scanner_pop3::do_login( )/*{{{*/
{
    mstring answer;
    if ( ! sock.Connected() )
    {
	return false;
    }
    if ( debuglevel > 1 )
    {
	cout << "\nSEND USER " << username << endl;
    }
    if ( sock.Send(mstring("USER ") + username + mstring("\r\n")) <= 0 )
    {
	sock.Disconnect();
	return false;
    }
    answer = get_answer_single();
    if ( command_count == LOGIN_TIME)
    {
	answer = answer.replace( username, "<USERNAME>" );
	scanned_return->add (answer);
	if ( debuglevel >= most )
	{
	    cout << "USER Result Checksum is : " << answer.strip_all_digits().checksum_sum() << endl;
	}
    }
    if ( debuglevel > 1 )
    {
	cout << "RECV " << answer << endl;
    }
    if ( ! sock.Connected() )
    {
	return false;
    }
    if ( debuglevel  > 1 )
    {
	cout << "\nSEND PASS " << password << endl;
    }
    if ( sock.Send(mstring("PASS ") + password + mstring("\r\n")) <= 0)
    {
	sock.Disconnect();
	return false;
    }
    answer = get_answer_single();
    if ( debuglevel > 1 )
    {
	cout << "RECV " << answer << endl;
//	cout << "RECV " << answer.strip_all_digits() << endl;
    }
    if ( command_count == LOGIN_TIME )
    {
	answer = answer.replace( username, "<USERNAME>" );
	scanned_return->add (answer.strip_all_digits());
	if ( debuglevel >= most )
	{
	    cout << "PASS Result Checksum is : " << answer.strip_all_digits().checksum_sum() << endl;
	}
    }
    if ( ! sock.Connected() )
    {
	return false;
    }
    return true;
}/*}}}*/

bool scanner_pop3::scan_special( long number )
{
    if (number == LOGIN_TIME)
    {
	sock.Receive((uint32)0,(uint32)0);
	if ( !sock.Connected() )
	{
	    sock.Reconnect();
	}
	if ( debuglevel >= most )
	{
	    cout << ". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . " << endl;
	}
	do_login(  );
	if ( debuglevel >= most )
	{
	    cout << ". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . " << endl;
	}
	sock.Receive((uint32)0,(uint32)0);
	if ( ! sock.Connected() )
	{
	    sock.Reconnect();
	    sock.Receive(10000);
	}
    }
    return true;
}

bool scanner_pop3::scan_reconnected( long number )
{
    mstring answer;
    answer = get_answer_single();
    if ( debuglevel >= most )
    {
	cout << endl << "DISCONNECT : " << answer << endl;
    }
    if ( number > LOGIN_TIME )
    {
	if ( debuglevel >= most )
	{
	    cout << ". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . " << endl;
	}
	do_login( );
	if ( debuglevel >= most )
	{
	    cout << ". . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . " << endl;
	}
    }
    return true;
}


bool scanner_pop3::scan_add_to_fp( const mstring&  answer )
{
    scanned_return->add( answer );
    return true;
}

bool scanner_pop3::scan_finished( void ) 
{
    return ( ! actual_command->command );
}

bool scanner_pop3::scan_init( void ) /*{{{*/
{
    mstring banner;
    if ( debuglevel >= every )
    {
	cout << "scanner_pop3::scan_init()" << endl;
    }
    actual_command = cmd_list;
    recon_count = 0;
    if ( debuglevel >= every )
    {
	cout << "sock.Connect(" << target_host << ", " << target_port << ", " << conn_timeout << ")" << endl;
    }
    if ( ! sock.Connect(target_host, target_port, conn_timeout) )
    {
	cout << " : " << sock.GetLastErrorMsg() << endl;
	return false;
    }
    banner = get_answer_single(); // banner
    if ( debuglevel >= every )
    {
	cout << "Creating local Data" << endl;
    }

    delete scanned_return;
    scanned_return = new fp_return(banner);
    return true;
}/*}}}*/

mstring scanner_pop3::is_server( void )
{
    return "unknown";
}
/**
 *$Log: scanner_pop3.cpp,v $
 *Revision 1.8  2003/03/27 10:58:17  plasmahh
 *new additions to stripping
 *
 *Revision 1.7  2003/03/13 17:30:45  plasmahh
 *Added some cool stripping stuff
 *Added pop3 scanning
 *
 *Revision 1.6  2003/02/25 23:23:46  plasmahh
 *fixed pop3 scanner bug
 *workaround for buggy rfc response servers
 *
 *Revision 1.5  2003/02/25 19:49:29  plasmahh
 *Ok, lets release ;)
 *
 *Revision 1.4  2003/02/25 16:36:35  plasmahh
 *A lot of changes ;)
 *
 *Revision 1.3  2003/01/27 16:53:52  plasmahh
 *migrating build system to athlon
 *
 *Revision 1.2  2003/01/07 13:36:22  plasmahh
 *more implementation of the pop3 protocol
 *
 *Revision 1.1  2002/12/19 23:18:23  plasmahh
 *first preliminary working pop3 support
 *changed options structure heavily
 *
 */
